.TH DALIB 3 "November, 1996" DALIB "Digital Audio Tape support Library"
.SH NAME
da_control \- Control the state of a DAT device
.SH SYNOPSIS
.B #include "dataudiolib.h"
.sp
.B int
.br
.BI "da_control( int datfd , int code [,void *arg] )"
.SH DESCRIPTION
The
.B da_control()
function controls the state of a datfiledescriptor and the associated
device. Depending on 
.B code
and 
.B arg
it initiates actions, returns or sets internal state information and the like.
.br
Possible values of code and their respective actions are:
.IP DA_GET_SAMPLERATE
Returns the current samplerate. arg is a 
.B "long*" 
pointing to a long integer where to samplerate will be stored.
.IP DA_SET_SAMPLERATE
Sets the samplerate of the datfiledescriptor to the value passed by arg.
arg is a 
.B "long*" 
pointing to a long integer containing the new samplerate. Possible values
are 44100, 48000 and 32000 (Hz). Using other values will cause da_control
to fail with EINVAL.
.IP DA_GET_CHANNELS
Gets the number of channels of an open datfiledescriptor. arg is 
.B "long*" 
pointing to a long integer containing the current number of channels.
.IP DA_SET_CHANNELS
Sets the number of channels to the value passed by arg. arg is a
.B "long*"
pointing to a long integer containing the new number of channels. Possible
values are 2 and 4. (Note that 4 is only valid using 32kHz packed format.)
.IP DA_GET_PACKED
Gets the packed-flag of an open datfiledescriptor. arg is a
.B "long*" 
pointing to a long integer containing either 0 or 1 (0 means not packed).
.IP DA_SET_PACKED
Sets the packed flag to the value passed by arg. arg is a
.B "long*"
pointing to a long integer containing the new packed-flag. Possible
values are 0 and 1. (Note that 1 is only valid using 32kHz samplerate.)
.IP DA_GET_PRERECORDED
Returns the current prerecorded state in a long integer pointed to by arg (
.B long *
). It will be either be 1 or 0. There is currently not much difference between
prerecorded and not prerecorded state.
.IP DA_SET_PRERECORDED
Sets the prerecorded state to the value pointed to by arg (
.B long*
).
.IP DA_GET_PROGNR
Gets the current programnumber of the datfiledescriptor. arg is a 
.B "long*"
pointing to a long integer, where the programnumber will be stored in. The 
programnumber will have been converted from BCD to integer.
.IP DA_SET_PROGNR
Sets the current programnumber to the value passed by arg. arg is a
.B "long*"
pointing to a long integer containing the new programnumber.
.IP DA_GET_PROGTIME
Returns the current program time in a 
.B struct da_time
pointed to by arg.
arg is a 
.B "struct da_time*"
which will be filled by da_control.
.IP DA_SET_PROGTIME
Sets the current programtime using the struct da_time pointed to by arg.
.IP DA_GET_ABSTIME
Gets the current absolute time in a 
.B struct da_time 
pointed to by arg. See above for the format of the struct da_time.
.IP DA_SET_ABSTIME
Sets the current absolute time using the struct da_time pointed to by arg.

.IP DA_GET_RECORDDATE
Gets the current record date associated with datfd into a 
.B struct da_date
pointed to by arg. 
.IP DA_SET_RECORDDATE
Sets the current recorddate of datfd. arg is a pointer to a
.B struct da_date 
with the new date.
.IP DA_GET_ISRC
Returns the current ISRC (International Standard Recording Code) of datfd in
a struct da_isrc pointed to by arg.
.IP DA_SET_ISRC
Sets the current ISRC of datfd. 
.B arg
is a
.B struct da_isrc*
pointing to a struct da_isrc.
.IP DA_GET_CATALOG
Returns the current catalog number (usually the EAN or UPC barcode of the sold
tape) in a
.B struct da_catalog 
pointed to by arg.
.IP DA_SET_CATALOG
Sets the current catalog number. 
.B arg is a
.B struct da_catalog*
pointing to the new information.
.IP DA_GET_TOCENTRY
Returns the TOC (table of contents) entry specified by 
.B prognr 
and 
.B value
in the 
.B struct da_tocentry
pointed to by arg. The other values of the struct will be filled on return.
.br
There are some special programnumber entries marking length of tape, beginning
etc. Please refer to the Postcript manual for their meanings. You may use
DA_TOC_AA,DA_TOC_BB,DA_TOC_CC,DA_TOC_EE,DA_TOC_B0,DA_TOC_A0,DA_TOC_A1,DA_TOC_C0
and DA_TOC_C1 to refer to those entries.
.IP DA_SET_TOCENTRY
Adds the TOC entry passed using arg as a
.B struct da_tocentry*
to the table of contents. The table of contents is added to the subcode area
while writing the tape.
.IP DA_ABORT
Abort current tape operation (like a running locate or rewind). This command
will generate a SCSI ABORT message to the SCSI tapedevice.
.IP DA_EJECT
Ejects the tape (if 
.B datfd
represents a real device). You should close the datfiledescriptor right after
that.
.IP DA_REWIND
Rewind the tape (if
.B datfd
represents a real device). 
.IP DA_WRITE_LEADIN
Writes a leadin zone on the datfiledescriptor.
.IP DA_WRITE_LEADOUT
Writes a leadout zone on the datfiledescriptor.
.IP DA_LOCATE
Seeks forward or backward to a position specified by the 
.B struct da_locate
pointed to by arg.
.B type
can be DA_LOC_PROGNR,DA_LOC_ABSTIME,DA_LOC_RECTIME or DA_LOC_PROGTIME. Except
for DA_LOG_PROGNR, the struct da_time part of the union will be used.
.br
DA_LOCATE behaves differently depending on the value of 
.B nextop
and if the datfiledescriptor was opened with the flag O_NDELAY. 
.B nextop
can be DA_LOC_NEXTOP_READ or DA_LOC_NEXTOP_WRITE (depending on which action
the caller wants to do next, if unsure, use DA_LOC_NEXTOP_WRITE).
.br
If the datfd was opened with O_NDELAY the library will start a LOCATE operation
and return immediately (with the tape still seeking). Any further reads 
will return immediately with a length of 0 bytes read until the tape has
stopped at the target position.
.br
A program number of 0xAAA will seek to the beginning of the tape, a program
number of 0xBBB will seek to the end of the tape.
.IP DA_READ_POSITION
Uses the SCSI command 
.B READ POSITION
to read the current position of the tape and stores it into the internal state
and returns it in a
.B struct da_position.
.IP DA_SET_TRACKNAME
Sets the trackname to the parameters passed in a
.B struct da_trackname.
The tracknames will be stored in a special format on the tape.
.IP DA_GET_TRACKNAME
Returns the trackname for the track specified by 
.B prognr 
in the 
.B struct da_trackname 
pointed to by arg.
.IP DA_GET_TRACKTIME
Returns the tracktime for the current track in a 
.B struct da_time
pointed to by 
.B arg.
.IP DA_SET_TRACKTIME 
Set the tracktime for this track using the
.B struct da_time
pointed to by 
.B arg.
.SH PARAMETERS
.SS Input Parameters
.IP datfd
A datfiledescriptor returned by either da_init or da_open.
.IP code
What action to do.
.IP arg
Depends on 
.B code
, usually a pointer to a struct or a long integer.
.SH RETURN VALUE
.IP arg
Depends on 
.B code
, usually a pointer to a struct or a long integer.
.br
When an error condition arises, this routine returns -1 and sets errno to
indicate the reason for the error.  Possible values of errno are:
.TP
EINVAL
The datfd is invalid.
.TP
other
A lot of other errors.
.SH USAGE EXAMPLE
.PP
.PP
.nf
		int datfd;

		datfd=da_init(fileno(stdin));
.fi
.SH SEE ALSO
.B da_intro(3),da_init(3)
.SH AUTHOR
Marcus Meissner <msmeissn@cip.informatik.uni-erlangen.de>
